@@ -19,12 +19,18 @@ import matplotlib.pyplot as plt
 
 @functools.lru_cache()
 def run_lowess(xnew, ynew, frac):
+    """
+    Internal function to run LOWESS.
+    """
     lowess = sm.nonparametric.lowess
     z = lowess(ynew, xnew, frac=frac, it=10)
     return z
 
 
 def array_to_tuple(np_array):
+    """
+    Internal function to convert array to tuple.
+    """
     try:
         return tuple(array_to_tuple(_) for _ in np_array)
     except TypeError:
@@ -32,6 +38,11 @@ def array_to_tuple(np_array):
 
 
 def get_elim_df_ordered(d, multi):
+    """
+    Internal function to extract elimination information from dictionary
+    and convert to data frame. Also performs log10 normalization on
+    features.
+    """
     if multi:
         col = 3
     else:
@@ -115,7 +126,7 @@ def extract_max_lowess(d, frac=3/10, multi=False, acc=False):
     return df_elim[(df_elim['log10_x'] == closest_val)].x.values[0], closest_val
 
 
-def extract_redundant_lowess(d, frac=3/10, step_size=0.05, multi=False,
+def extract_redundant_lowess(d, frac=3/10, step_size=0.02, multi=False,
                              acc=False):
     """
     Extract redundant features based on rate of change of log10
@@ -125,7 +136,10 @@ def extract_redundant_lowess(d, frac=3/10, step_size=0.05, multi=False,
     d: Dictionary from dRFE
     frac: Fraction for lowess smoothing. Default 3/10.
     step_size: Rate of change step size to analyze for extraction
-    (default: 0.05)
+    (default: 0.02)
+    multi: Is the target multi-class (boolean). Default False.
+    classify: Is the target classification (boolean). Default True.
+    acc: Use accuracy metric to optimize data (boolean). Default False.
 
     Yields:
     int: number of redundant features
@@ -150,8 +164,28 @@ def extract_redundant_lowess(d, frac=3/10, step_size=0.05, multi=False,
     return redundant_feat, redunt_feat_log10
 
 
-def optimize_lowess_plot(d, fold, output_dir, frac=3/10, step_size=0.05,
-                         classify=True, save_plot=False, multi=False, acc=False):
+def optimize_lowess_plot(d, fold, output_dir, frac=3/10, step_size=0.02,
+                         classify=True, save_plot=False, multi=False, acc=False,
+                         log=True, print_out=True):
+    """
+    Plot the LOWESS smoothing plot for RFE with lines annotating set selection.
+
+    Args:
+    d: Dictionary from dRFE
+    frac: Fraction for lowess smoothing. Default 3/10.
+    step_size: Rate of change step size to analyze for extraction
+    (default: 0.02)
+    multi: Is the target multi-class (boolean). Default False.
+    classify: Is the target classification (boolean). Default True.
+    acc: Use accuracy metric to optimize data (boolean). Default False.
+    save_plot: Save the optmization plot (boolean). Default False.
+    log: Plot with log transformation (boolean). Default True.
+    print_out: Print to screen (boolean). Default True.
+
+    Yields:
+    Plot of the data with annotation and LOWESS smoothing.
+    Saves plot if specified.
+    """
     if classify:
         if multi:
             label = 'ROC AUC'
@@ -162,11 +196,14 @@ def optimize_lowess_plot(d, fold, output_dir, frac=3/10, step_size=0.05,
     else:
         label = 'R2'
     title = 'Fraction: %.2f, Step Size: %.2f' % (frac, step_size)
-    _, max_feat_log10 = extract_max_lowess(d, frac, multi, acc)
     x,y,z,_,_ = cal_lowess(d, frac, multi, acc)
     df_elim = pd.DataFrame({'X': x, 'Y': y})
-    _,lo = extract_max_lowess(d, frac, multi, acc)
-    _,l1 = extract_redundant_lowess(d, frac, step_size, multi, acc)
+    if log:
+        _,lo = extract_max_lowess(d, frac, multi, acc)
+        _,l1 = extract_redundant_lowess(d, frac, step_size, multi, acc)
+    else:
+        lo,_ = extract_max_lowess(d, frac, multi, acc)
+        l1,_ = extract_redundant_lowess(d, frac, step_size, multi, acc)
     plt.clf()
     f1 = plt.figure()
     plt.plot(x, y, 'o', label="dRFE")
@@ -176,11 +213,13 @@ def optimize_lowess_plot(d, fold, output_dir, frac=3/10, step_size=0.05,
     plt.vlines(l1, ymin=np.min(y), ymax=np.max(y),
                colors='orange', linestyles='--',
                label='Redundant Features')
-    plt.xscale('log')
-    plt.xlabel('log10(N Features)')
-    plt.ylabel(label)
-    plt.legend(loc='best')
-    plt.title(title)
+    if log:
+        plt.xscale('log')
+        plt.xlabel('log(N Features)')
+    else:
+        plt.xscale('linear')
+        plt.xlabel('N Features')        
+    plt.ylabel(label); plt.legend(loc='best'); plt.title(title)
     if save_plot:
         plt.savefig("%s/optimize_lowess_%s_frac%.2f_step_%.2f_%s.png" %
                     (output_dir, fold, frac, step_size, label.replace(" ", "_")))
@@ -188,4 +227,5 @@ def optimize_lowess_plot(d, fold, output_dir, frac=3/10, step_size=0.05,
                     (output_dir, fold, frac, step_size, label.replace(" ", "_")))
         plt.savefig("%s/optimize_lowess_%s_frac%.2f_step_%.2f_%s.svg" %
                     (output_dir, fold, frac, step_size, label.replace(" ", "_")))
-    plt.show()
+    if print_out:
+        plt.show()
